import { FrameRequestCallback } from './types';
export declare type RequestFrameFn = (cb: FrameRequestCallback) => number | void;
export interface OpaqueAnimation {
    idle: boolean;
    priority: number;
    advance(dt: number): void;
}
/** Create a frameloop singleton */
export declare class FrameLoop {
    idle: boolean;
    /**
     * Start a new animation, or reorder an active animation in
     * the animations array in response to a priority change.
     */
    start: (animation: OpaqueAnimation) => void;
    /**
     * Update every active animation.
     *
     * Can be passed to `requestAnimationFrame` without wrapping or binding.
     */
    update: (time?: number) => void;
    /**
     * Execute a function once after all animations have updated.
     */
    onFrame: (cb: FrameRequestCallback) => void;
    /**
     * Execute a function once at the very end of the current frame.
     *
     * Only call this within an `onFrame` callback.
     */
    onWrite: (cb: FrameRequestCallback) => void;
    constructor(requestFrame?: RequestFrameFn);
}
