import { ElementType } from 'react';
import { FluidValue } from 'fluids';
import { InterpolatorConfig, InterpolatorArgs, OneOrMore } from './types';
import { FrameLoop } from './FrameLoop';
declare const window: {
    requestAnimationFrame: (cb: (time: number) => void) => number;
    cancelAnimationFrame: (id: number) => void;
};
export declare let defaultElement: string | ElementType;
export declare let applyAnimatedValues: (node: any, props: any) => boolean | void;
export declare let createStringInterpolator: (config: InterpolatorConfig<string>) => (input: number) => string;
export declare let frameLoop: FrameLoop;
export declare let to: <In, Out>(source: OneOrMore<FluidValue>, args: InterpolatorArgs<In, Out>) => FluidValue<Out>;
export declare let now: () => number;
export declare let performanceNow: () => number;
export declare let colorNames: {
    [key: string]: number;
} | null;
export declare let skipAnimation: boolean;
export declare let getComponentProps: (props: any) => any;
export declare let createAnimatedStyle: ((style: any) => any) | null;
export declare let createAnimatedTransform: ((transform: any) => any) | null;
export declare let requestAnimationFrame: typeof window.requestAnimationFrame;
export declare let cancelAnimationFrame: typeof window.cancelAnimationFrame;
export declare let batchedUpdates: (callback: () => void) => void;
export interface AnimatedGlobals {
    /** Returns a new `Interpolation` object */
    to?: typeof to;
    /** Used for timestamps in milliseconds. Read more [here](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/now) */
    now?: typeof now;
    /** Provide a custom `FrameLoop` instance */
    frameLoop?: typeof frameLoop;
    /** Provide custom color names for interpolation */
    colorNames?: typeof colorNames;
    /** Make all animations instant and skip the frameloop entirely */
    skipAnimation?: typeof skipAnimation;
    /** The `div` element equivalent for the current platform */
    defaultElement?: typeof defaultElement;
    /** Used to measure frame length. Read more [here](https://developer.mozilla.org/en-US/docs/Web/API/Performance/now) */
    performanceNow?: typeof performanceNow;
    /** Intercept props before they're passed to an animated component */
    getComponentProps?: typeof getComponentProps;
    /** Provide custom logic for native updates */
    applyAnimatedValues?: typeof applyAnimatedValues;
    /** Provide custom logic for string interpolation */
    createStringInterpolator?: typeof createStringInterpolator;
    /** Wrap the `transform` prop with an animated node */
    createAnimatedTransform?: typeof createAnimatedTransform;
    /** Wrap the `style` prop with an animated node */
    createAnimatedStyle?: typeof createAnimatedStyle;
    /** Schedule a function to run on the next frame */
    requestAnimationFrame?: typeof requestAnimationFrame;
    /** Prevent a scheduled function from running on the next frame */
    cancelAnimationFrame?: typeof cancelAnimationFrame;
    /** Event props are called with `batchedUpdates` to reduce extraneous renders */
    batchedUpdates?: typeof batchedUpdates;
}
export declare const assign: (globals: AnimatedGlobals) => AnimatedGlobals;
export {};
